#! /usr/bin/python

# module to get all the file types in a given directory:
import fnmatch

import os
import sys
import time
import datetime


# a function to locate php files recursively in directories 
def locate(pattern, root=os.curdir):
    for path, dirs, files in os.walk(os.path.abspath(root)):
        for filename in (filename for filename in files if fnmatch.fnmatch(filename, pattern)):
            yield os.path.join(path, filename)

# setup analysis parameters. 
# To run more than one analyses separate with colon i.e. xssstranger:sqlstranger:mfestranger. 
# Do the same for attack patterns. If you do not have an attack pattern then use empty like this //
# Note for attack patterns: you need to escape stranger regExp special chars with a back slash. 
# Then you need to escape this packslash with another for string processing by python (so always escape with 2 backslashes)
analyses = 'xssstranger:sqlstranger:mfestranger'
attackptrn = "/.*\\<SCRIPT .*\\>.*/:/.*' or 1=1 '.*/:/.*/evil.*/"

# setup path parameters for stranger
mypath = os.getcwd()
strangerPath = mypath+'/lib'
datetimevalue = datetime.datetime.now().strftime("%m-%d-%y_%H:%M:%S") 
outputdirbase = "."

#class path is useless with jar files ???!!! according to java docs. Already specified in the jar manifest file
#classpath = mypath+"/lib"

# execution time
total_time = 0.0

# read cmd arguments
if len(sys.argv) < 2:
    print "usage: test-php-app.py [output-directory] php-app-top-dir-path"
    sys.exit()
if len(sys.argv) < 3:
    appPath = sys.argv[1]
else:
    outputdirbase = sys.argv[1]
    appPath = sys.argv[2]

# create top output directory for the result
topoutputdir = outputdirbase + "/" + os.path.split(appPath)[1] + "_" + datetimevalue
os.mkdir(topoutputdir,0770)

# run stranger for each php script in the web application
for phpfile in locate("*.php", appPath):    

    # generate output directory for a php script within the top directory
    srcFilename = os.path.split(phpfile)[1]
    outputdir = topoutputdir + "/" + srcFilename + ".result"
    while os.path.exists(outputdir):
        outputdir = outputdir + "_"
    os.mkdir(outputdir,0770)
    
    # copy php script file to the output directory (make it easier to analyze result)
    os.system ("cp %s %s" % (phpfile, outputdir + "/" + srcFilename))
    
    # start timer
    t0 = time.time()
    
    # run stranger
    # You need to balance memory between java preliminary analyses for PHP (such as literal analysis) and string analysis (done by c library) 
    # by giving java the amount of memory you think it needs using the minimum (-Xms) and maximum (-Xmx) parameters.
    # Note that memory consumed by c library is not replicated in Java (by jna) so memory needed by string analysis 
    # will only be consumed by c library (This is why memory given to java here is so small :-)
    os.system('java -Xms256m -Xmx1024m -Dpixy.home="'+mypath+'" -Djna.library.path="'+strangerPath+'" -jar stranger.jar -o '+outputdir+' -a -g -W 5 -C 40 -y '+ analyses + ' -k "' + attackptrn + '" ' + phpfile)
    
    # stop timer
    total_time = total_time + (time.time() - t0)
    
    # This would remove the output directory for files that do not contain tainted sinks
    empty = True 
    for dotfile in locate("*.dot", outputdir):
        empty = False
	break
        #os.system("dot -Tpng "+dotfile+" -o "+dotfile+".png") #uncomment to convert all dot files into png - WARNING costs lots of time -
    if empty:
        os.system("rm -r %s" % (outputdir))

print "======================================================="
print "Total analysis time for application \"", os.path.split(appPath)[1], "\": ", total_time, " seconds"
print "=======================================================\n\n"
