#! /usr/bin/python

# module to get all the file types in a given directory:
import fnmatch

import os
import sys
import time
import datetime


# a function to locate php files recursively in directories 
def locate(pattern, root=os.curdir):
    for path, dirs, files in os.walk(os.path.abspath(root)):
        for filename in (filename for filename in files if fnmatch.fnmatch(filename, pattern)):
            yield os.path.join(path, filename)

# setup analysis parameters. 
# To run more than one analyses separate with colon i.e. xssstranger:sqlstranger:mfestranger. 
# Do the same for attack patterns. If you do not have an attack pattern then use empty like this //
# Note for attack patterns: you need to escape stranger regExp special chars with a back slash. 
# Then you need to escape this packslash with another for string processing by python (so always escape with 2 backslashes)
analyses = 'xssstranger:sqlstranger:mfestranger'
attackptrn = "/.*\\<SCRIPT .*\\>.*/:/.*' or 1=1 '.*/:/.*/evil.*/"
            
# setup path parameters for stranger
mypath = os.getcwd()
strangerPath = mypath+'/lib'
outputdirbase = "."


#class path is useless with jar files ???!!! according to java docs. Already specified in the jar manifest file
#classpath = mypath+"/lib:"+mypath+"/build/class:"+mypath+"/../java-stranger-connector/jna.jar:"+mypath+"/../java-stranger-connector/bin"




# read cmd arguments
if len(sys.argv) < 2:
    print "usage: test-php-script.py [output-directory] php-script-fqn-name"
    sys.exit()
if len(sys.argv) < 3:
    scriptName = sys.argv[1]
else:
    outputdirbase = sys.argv[1]
    scriptName = sys.argv[2]

# create the output directory
datetimevalue = datetime.datetime.now().strftime("%m-%d-%y_%H:%M:%S") 
outputdir = outputdirbase + "/" + os.path.split(scriptName)[1] + "_" + datetimevalue
os.mkdir(outputdir,0770)

# run stranger
# You need to balance memory between java preliminary analyses for PHP (such as literal analysis) and string analysis (done by c library) 
# by giving java the amount of memory you think it needs using the minimum (-Xms) and maximum (-Xmx) parameters.
# Note that memory consumed by c library is not replicated in Java (by jna) so memory needed by string analysis 
# will only be consumed by c library (This is why memory given to java here is so small :-)
os.system('java -Xms32m -Xmx256m -Dpixy.home="'+mypath+'" -Djna.library.path="'+strangerPath+'" -jar stranger.jar -o '+outputdir+' -a -g -W 5 -C 40 -B -y '+ analyses + ' -k "' + attackptrn + '" ' + scriptName)

# convert dot files to png
for dotfile in locate("*.dot", outputdir):
    os.system("dot -Tpng "+dotfile+" -o "+dotfile+".png")
   
